@@ -61,6 +61,7 @@ _tERMPAINT_PUBLIC termpaint_terminal *termpaint_terminal_new(termpaint_integrati
 _tERMPAINT_PUBLIC termpaint_terminal *termpaint_terminal_new_or_nullptr(termpaint_integration *integration);
 _tERMPAINT_PUBLIC void termpaint_terminal_free(termpaint_terminal *term);
 _tERMPAINT_PUBLIC void termpaint_terminal_free_with_restore(termpaint_terminal *term);
+_tERMPAINT_PUBLIC void termpaint_terminal_free_with_restore_and_persistent(termpaint_terminal *term, termpaint_surface *surface);
 _tERMPAINT_PUBLIC termpaint_surface *termpaint_terminal_get_surface(termpaint_terminal *term);
 _tERMPAINT_PUBLIC void termpaint_terminal_flush(termpaint_terminal *term, _Bool full_repaint);
 _tERMPAINT_PUBLIC const char *termpaint_terminal_restore_sequence(const termpaint_terminal *term);
@@ -129,6 +130,8 @@ _tERMPAINT_PUBLIC void termpaint_terminal_auto_detect_apply_input_quirks(termpai
 _tERMPAINT_PUBLIC void termpaint_terminal_auto_detect_result_text(const termpaint_terminal *terminal, char *buffer, int buffer_length);
 _tERMPAINT_PUBLIC const char *termpaint_terminal_self_reported_name_and_version(const termpaint_terminal *terminal);
 _tERMPAINT_PUBLIC void termpaint_terminal_setup_fullscreen(termpaint_terminal *terminal, int width, int height, const char *options);
+_tERMPAINT_PUBLIC void termpaint_terminal_setup_inline(termpaint_terminal *terminal, int width, int height, const char *options);
+_tERMPAINT_PUBLIC void termpaint_terminal_set_inline(termpaint_terminal *terminal, _Bool enabled);
 
 #define TERMPAINT_CAPABILITY_SAFE_POSITION_REPORT 0
 #define TERMPAINT_CAPABILITY_CSI_GREATER 1
@@ -153,6 +156,7 @@ _tERMPAINT_PUBLIC void termpaint_terminal_disable_capability(termpaint_terminal
 _tERMPAINT_PUBLIC _Bool termpaint_terminal_should_use_truecolor(termpaint_terminal *terminal);
 
 _tERMPAINT_PUBLIC void termpaint_terminal_pause(termpaint_terminal *term);
+_tERMPAINT_PUBLIC void termpaint_terminal_pause_and_persistent(termpaint_terminal *term, termpaint_surface *surface);
 _tERMPAINT_PUBLIC void termpaint_terminal_unpause(termpaint_terminal *term);
 
 #define TERMPAINT_LOG_AUTO_DETECT_TRACE (1 << 0)
