@@ -37,12 +37,16 @@
 typedef struct termpaint_integration_fd_ {
     termpaint_integration base;
     char *options;
-    int fd;
-    bool auto_close;
+    // In rare situations, read and write MUST use different fds to be able to communicate with the terminal
+    int fd_read;
+    int fd_write;
+    bool auto_close; // if true fd_read == fd_write is assumed
     struct termios original_terminal_attributes;
     bool callback_requested;
     bool awaiting_response;
     bool poll_sigwinch;
+    bool inline_active;
+    int inline_height;
     termpaint_terminal *terminal;
     termpaintx_ttyrescue *rescue;
 } termpaint_integration_fd;
@@ -91,6 +95,33 @@ static bool termpaintp_is_file_rw(int fd) {
     return ret != -1 && (ret & O_ACCMODE) == O_RDWR;
 }
 
+static bool termpaintp_is_file_readable(int fd) {
+    int ret = fcntl(fd, F_GETFL);
+    return ret != -1 && ((ret & O_ACCMODE) == O_RDWR || (ret & O_ACCMODE) == O_RDONLY);
+}
+
+static bool termpaintp_is_file_writable(int fd) {
+    int ret = fcntl(fd, F_GETFL);
+    return ret != -1 && ((ret & O_ACCMODE) == O_RDWR || (ret & O_ACCMODE) == O_WRONLY);
+}
+
+static bool termpaintp_is_terminal_fd_pair(int fd_read, int fd_write) {
+    if (isatty(fd_read) && isatty(fd_write)
+            && termpaintp_is_file_readable(fd_read) && termpaintp_is_file_writable(fd_write)) {
+        struct stat statbuf_r;
+        struct stat statbuf_w;
+
+        if (fstat(fd_read, &statbuf_r) == 0 && fstat(fd_write, &statbuf_w) == 0) {
+            if (statbuf_r.st_rdev == statbuf_w.st_rdev
+                    && statbuf_r.st_dev == statbuf_w.st_dev
+                    && statbuf_r.st_ino == statbuf_w.st_ino) {
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
 _Bool termpaintx_full_integration_available(void) {
     _Bool from_std_fd = false;
     from_std_fd = (isatty(0) && termpaintp_is_file_rw(0))
@@ -105,12 +136,24 @@ _Bool termpaintx_full_integration_available(void) {
         close(fd);
         return true;
     }
+
+    // as last resort check if (0, 1) or (0, 2) are matching read/write pairs pointing to the same terminal
+    if (termpaintp_is_terminal_fd_pair(0, 1)) {
+        return true;
+    }
+
+    if (termpaintp_is_terminal_fd_pair(0, 2)) {
+        return true;
+    }
+
     return false;
 }
 
 termpaint_integration *termpaintx_full_integration(const char *options) {
     int fd = -1;
+    int fd_write = -1; // only if differs
     _Bool auto_close = false;
+    bool might_be_controlling_terminal = true;
 
     if (isatty(0) && termpaintp_is_file_rw(0)) {
         fd = 0;
@@ -120,15 +163,30 @@ termpaint_integration *termpaintx_full_integration(const char *options) {
         fd = 2;
     } else {
         fd = open("/dev/tty", O_RDWR | O_NOCTTY | FD_CLOEXEC);
-        auto_close = true;
-        if (fd == -1) {
-            return nullptr;
+        if (fd != -1) {
+            auto_close = true;
+        } else {
+            might_be_controlling_terminal = false;
+            // as last resort check if (0, 1) or (0, 2) are matching read/write pairs pointing to the same terminal
+
+            if (termpaintp_is_terminal_fd_pair(0, 1)) {
+                fd = 0;
+                fd_write = 1;
+            } else if (termpaintp_is_terminal_fd_pair(0, 2)) {
+                fd = 0;
+                fd_write = 2;
+            } else {
+                return nullptr;
+            }
         }
     }
 
-    termpaint_integration *ret = termpaintx_full_integration_from_fd(fd, auto_close, options);
-    termpaintp_setup_winch();
-    FDPTR(ret)->poll_sigwinch = true;
+    termpaint_integration *ret = (fd_write == -1) ? termpaintx_full_integration_from_fd(fd, auto_close, options)
+                                                  : termpaintx_full_integration_from_fds(fd, fd_write, options);
+    if (might_be_controlling_terminal) {
+        termpaintp_setup_winch();
+        FDPTR(ret)->poll_sigwinch = true;
+    }
     return ret;
 }
 
@@ -165,12 +223,12 @@ static void fd_free(termpaint_integration* integration) {
             }
             int ret;
             struct pollfd info;
-            info.fd = fd_data->fd;
+            info.fd = fd_data->fd_read;
             info.events = POLLIN;
             ret = poll(&info, 1, 100 - time_waited_ms);
             if (ret == 1) {
                 char buff[1000];
-                int amount = (int)read(fd_data->fd, buff, 999);
+                int amount = (int)read(fd_data->fd_read, buff, 999);
                 if (amount < 0) {
                     break;
                 }
@@ -183,9 +241,10 @@ static void fd_free(termpaint_integration* integration) {
         fd_data->rescue = nullptr;
     }
 
-    tcsetattr (fd_data->fd, TCSAFLUSH, &fd_data->original_terminal_attributes);
-    if (fd_data->auto_close && fd_data->fd != -1) {
-        close(fd_data->fd);
+    tcsetattr (fd_data->fd_read, TCSAFLUSH, &fd_data->original_terminal_attributes);
+    if (fd_data->auto_close && fd_data->fd_read != -1) {
+        // assumnes that auto_close will only be true if fd_read == fd_write
+        close(fd_data->fd_read);
     }
     free(fd_data->options);
     termpaint_integration_deinit(&fd_data->base);
@@ -198,19 +257,20 @@ static void fd_flush(termpaint_integration* integration) {
 }
 
 static void fd_mark_bad(termpaint_integration* integration) {
-    FDPTR(integration)->fd = -1;
+    FDPTR(integration)->fd_read = -1;
+    FDPTR(integration)->fd_write = -1;
 }
 
 static _Bool fd_is_bad(termpaint_integration* integration) {
-    return FDPTR(integration)->fd == -1;
+    return FDPTR(integration)->fd_read == -1;
 }
 
-static void fd_write(termpaint_integration* integration, const char *data, int length) {
+static void fd_write_data(termpaint_integration* integration, const char *data, int length) {
     ssize_t written = 0;
     ssize_t ret;
     errno = 0;
     while (written != length) {
-        ret = write(FDPTR(integration)->fd, data+written, length-written);
+        ret = write(FDPTR(integration)->fd_write, data+written, length-written);
         if (ret > 0) {
             written += ret;
         } else {
@@ -303,23 +363,37 @@ bool termpaintx_fd_set_termios(int fd, const char *options) {
     return termpaintp_fd_set_termios(fd, options);
 }
 
-termpaint_integration *termpaintx_full_integration_from_fd(int fd, _Bool auto_close, const char *options) {
+static termpaint_integration *termpaintp_full_integration_from_fds(int fd_read, int fd_write, _Bool auto_close, const char *options) {
+    // NOTE: If fd_read != fd_write then auto_close must be false.
     termpaint_integration_fd *ret = calloc(1, sizeof(termpaint_integration_fd));
-    termpaint_integration_init(&ret->base, fd_free, fd_write, fd_flush);
+    if (!ret) {
+        (void)!write(2, "Out of memory\n", 14); // ignore error, exiting soon anyway.
+        abort();
+    }
+    termpaint_integration_init(&ret->base, fd_free, fd_write_data, fd_flush);
     termpaint_integration_set_is_bad(&ret->base, fd_is_bad);
     termpaint_integration_set_request_callback(&ret->base, fd_request_callback);
     termpaint_integration_set_awaiting_response(&ret->base, fd_awaiting_response);
     ret->options = strdup(options);
-    ret->fd = fd;
+    ret->fd_read = fd_read;
+    ret->fd_write = fd_write;
     ret->auto_close = auto_close;
     ret->callback_requested = false;
     ret->awaiting_response = false;
 
-    tcgetattr(ret->fd, &ret->original_terminal_attributes);
-    termpaintp_fd_set_termios(ret->fd, options);
+    tcgetattr(ret->fd_read, &ret->original_terminal_attributes);
+    termpaintp_fd_set_termios(ret->fd_read, options);
     return (termpaint_integration*)ret;
 }
 
+termpaint_integration *termpaintx_full_integration_from_fd(int fd, _Bool auto_close, const char *options) {
+    return termpaintp_full_integration_from_fds(fd, fd, auto_close, options);
+}
+
+termpaint_integration *termpaintx_full_integration_from_fds(int fd_read, int fd_write, const char *options) {
+    return termpaintp_full_integration_from_fds(fd_read, fd_write, false, options);
+}
+
 void termpaintx_full_integration_wait_for_ready(termpaint_integration *integration) {
     termpaint_integration_fd *t = FDPTR(integration);
     while (termpaint_terminal_auto_detect_state(t->terminal) == termpaint_auto_detect_running) {
@@ -339,7 +413,7 @@ void termpaintx_full_integration_wait_for_ready_with_message(termpaint_integrati
                 break;
             }
             if (milliseconds <= 0) {
-                fd_write(integration, message, strlen(message));
+                fd_write_data(integration, message, strlen(message));
             }
         } else {
             if (!termpaintx_full_integration_do_iteration(integration)) {
@@ -351,10 +425,10 @@ void termpaintx_full_integration_wait_for_ready_with_message(termpaint_integrati
 }
 
 bool termpaintx_full_integration_terminal_size(termpaint_integration *integration, int *width, int *height) {
-    if (fd_is_bad(integration) || !isatty(FDPTR(integration)->fd)) {
+    if (fd_is_bad(integration) || !isatty(FDPTR(integration)->fd_read)) {
         return false;
     }
-    return termpaintx_fd_terminal_size(FDPTR(integration)->fd, width, height);
+    return termpaintx_fd_terminal_size(FDPTR(integration)->fd_read, width, height);
 }
 
 void termpaintx_full_integration_set_terminal(termpaint_integration *integration, termpaint_terminal *terminal) {
@@ -372,9 +446,13 @@ static void termpaintp_handle_self_pipe(termpaint_integration_fd *t, struct poll
             sigwinch_set = false;
         }
         int width, height;
-        termpaintx_full_integration_terminal_size(&t->base, &width, &height);
-        termpaint_surface* surface = termpaint_terminal_get_surface(t->terminal);
-        termpaint_surface_resize(surface, width, height);
+        if (termpaintx_full_integration_terminal_size(&t->base, &width, &height)) {
+            if (t->inline_active && t->inline_height && height > t->inline_height) {
+                height = t->inline_height;
+            }
+            termpaint_surface* surface = termpaint_terminal_get_surface(t->terminal);
+            termpaint_surface_resize(surface, width, height);
+        }
     } else {
         // something broken, don't try again
         sigwinch_set = false;
@@ -387,7 +465,7 @@ bool termpaintx_full_integration_do_iteration(termpaint_integration *integration
     char buff[1000];
     if (t->poll_sigwinch && sigwinch_set) {
         struct pollfd info[2];
-        info[0].fd = t->fd;
+        info[0].fd = t->fd_read;
         info[0].events = POLLIN;
         info[1].fd = sigwinch_pipe[0];
         info[1].events = POLLIN;
@@ -400,7 +478,7 @@ bool termpaintx_full_integration_do_iteration(termpaint_integration *integration
             return true;
         }
     }
-    int amount = (int)read(t->fd, buff, 999);
+    int amount = (int)read(t->fd_read, buff, 999);
     if (amount < 0) {
         if (errno != EINTR && errno != EWOULDBLOCK) {
             return false;
@@ -414,11 +492,11 @@ bool termpaintx_full_integration_do_iteration(termpaint_integration *integration
     if (t->callback_requested) {
         t->callback_requested = false;
         struct pollfd info;
-        info.fd = t->fd;
+        info.fd = t->fd_read;
         info.events = POLLIN;
         int ret = poll(&info, 1, 100);
         if (ret == 1) {
-            int amount = (int)read(t->fd, buff, 999);
+            int amount = (int)read(t->fd_read, buff, 999);
             if (amount < 0) {
                 if (errno != EINTR && errno != EWOULDBLOCK) {
                     return false;
@@ -448,7 +526,7 @@ bool termpaintx_full_integration_do_iteration_with_timeout(termpaint_integration
     {
         int count = 1;
         struct pollfd info[2];
-        info[0].fd = t->fd;
+        info[0].fd = t->fd_read;
         info[0].events = POLLIN;
         if (t->poll_sigwinch && sigwinch_set) {
             info[1].fd = sigwinch_pipe[0];
@@ -473,7 +551,7 @@ bool termpaintx_full_integration_do_iteration_with_timeout(termpaint_integration
         }
     }
     if (ret == 1) {
-        int amount = (int)read(t->fd, buff, 999);
+        int amount = (int)read(t->fd_read, buff, 999);
         if (amount < 0) {
             if (errno != EINTR && errno != EWOULDBLOCK) {
                 return false;
@@ -494,12 +572,12 @@ bool termpaintx_full_integration_do_iteration_with_timeout(termpaint_integration
                        + now.tv_nsec / 1000000 - start_time.tv_nsec / 1000000);
             if (remaining > 0) {
                 struct pollfd info;
-                info.fd = t->fd;
+                info.fd = t->fd_read;
                 info.events = POLLIN;
                 ret = poll(&info, 1, remaining < 100 ? remaining : 100);
             }
             if (ret == 1) {
-                int amount = (int)read(t->fd, buff, 999);
+                int amount = (int)read(t->fd_read, buff, 999);
                 if (amount < 0) {
                     if (errno != EINTR && errno != EWOULDBLOCK) {
                         return false;
@@ -554,7 +632,7 @@ static void fd_restore_sequence_updated(struct termpaint_integration_ *integrati
 bool termpaintx_full_integration_ttyrescue_start(termpaint_integration *integration) {
     termpaint_integration_fd *t = FDPTR(integration);
     if (t->rescue || !t->terminal) return false;
-    t->rescue = termpaintx_ttyrescue_start_or_nullptr(t->fd, termpaint_terminal_restore_sequence(t->terminal));
+    t->rescue = termpaintx_ttyrescue_start_or_nullptr(t->fd_write, termpaint_terminal_restore_sequence(t->terminal));
     if (t->rescue) {
         termpaint_integration_set_restore_sequence_updated(integration, fd_restore_sequence_updated);
         termpaintx_ttyrescue_set_restore_termios(t->rescue, &t->original_terminal_attributes);
@@ -563,8 +641,7 @@ bool termpaintx_full_integration_ttyrescue_start(termpaint_integration *integrat
     return false;
 }
 
-
-termpaint_integration *termpaintx_full_integration_setup_terminal_fullscreen(const char *options,
+static termpaint_integration *termpaintp_full_integration_setup_terminal_common(const char *options,
                                                                              void (*event_handler)(void *, termpaint_event *),
                                                                              void *event_handler_user_data,
                                                                              termpaint_terminal **terminal_out) {
@@ -582,7 +659,24 @@ termpaint_integration *termpaintx_full_integration_setup_terminal_fullscreen(con
     termpaintx_full_integration_wait_for_ready_with_message(integration, 10000,
                                            "Terminal auto detection is taking unusually long, press space to abort.");
     termpaintx_full_integration_apply_input_quirks(integration);
-    int width, height;
+    *terminal_out = terminal;
+    return integration;
+}
+
+termpaint_integration *termpaintx_full_integration_setup_terminal_fullscreen(const char *options,
+                                                                             void (*event_handler)(void *, termpaint_event *),
+                                                                             void *event_handler_user_data,
+                                                                             termpaint_terminal **terminal_out) {
+    termpaint_terminal *terminal;
+    termpaint_integration *integration = termpaintp_full_integration_setup_terminal_common(options,
+                                                                                           event_handler,
+                                                                                           event_handler_user_data,
+                                                                                           &terminal);
+    if (!integration) {
+        return nullptr;
+    }
+
+    int width = 80, height = 25;
     termpaintx_full_integration_terminal_size(integration, &width, &height);
     termpaint_terminal_setup_fullscreen(terminal, width, height, options);
     termpaintx_full_integration_ttyrescue_start(integration);
@@ -590,6 +684,55 @@ termpaint_integration *termpaintx_full_integration_setup_terminal_fullscreen(con
     return integration;
 }
 
+
+termpaint_integration *termpaintx_full_integration_setup_terminal_inline(const char *options,
+                                                                         int lines,
+                                                                         void (*event_handler)(void *, termpaint_event *),
+                                                                         void *event_handler_user_data,
+                                                                         termpaint_terminal **terminal_out) {
+    termpaint_terminal *terminal;
+    termpaint_integration *integration = termpaintp_full_integration_setup_terminal_common(options,
+                                                                                           event_handler,
+                                                                                           event_handler_user_data,
+                                                                                           &terminal);
+    if (!integration) {
+        return nullptr;
+    }
+    termpaint_integration_fd* fd_data = FDPTR(integration);
+    fd_data->inline_height = lines;
+    fd_data->inline_active = true;
+    int width = 80, height = 25;
+    termpaintx_full_integration_terminal_size(integration, &width, &height);
+    if (height > lines) {
+        height = lines;
+    }
+    termpaint_terminal_setup_inline(terminal, width, height, options);
+    termpaintx_full_integration_ttyrescue_start(integration);
+    *terminal_out = terminal;
+    return integration;
+}
+
+void termpaintx_full_integration_set_inline(termpaint_integration *integration, _Bool enabled, int height) {
+    termpaint_integration_fd* fd_data = FDPTR(integration);
+
+    if (height > 0) {
+        fd_data->inline_height = height;
+    }
+
+    termpaint_terminal_set_inline(fd_data->terminal, enabled);
+    fd_data->inline_active = enabled;
+
+    int term_width, term_height;
+    if (termpaintx_full_integration_terminal_size(integration, &term_width, &term_height)) {
+        if (fd_data->inline_active && fd_data->inline_height && term_height > fd_data->inline_height) {
+            term_height = fd_data->inline_height;
+        }
+        termpaint_surface* surface = termpaint_terminal_get_surface(fd_data->terminal);
+        termpaint_surface_resize(surface, term_width, term_height);
+    }
+}
+
+
 static void termpaintx_dummy_log(struct termpaint_integration_ *integration, char *data, int length) {
     (void)integration;
     (void)data;
