@@ -6,7 +6,11 @@
 #include <numeric>
 
 #ifndef BUNDLED_CATCH2
+#ifdef CATCH3
+#include "catch2/catch_all.hpp"
+#else
 #include "catch2/catch.hpp"
+#endif
 #else
 #include "../third-party/catch.hpp"
 #endif
@@ -56,6 +60,18 @@ private:
     iconv_t cache;
 };
 
+#ifdef __NetBSD__
+#include <sys/param.h>
+#if __NetBSD_Version__ < 1000000000
+// NetBSD < 10 does not follow posix and takes a const.
+// See https://man.netbsd.org/iconv.3#STANDARDS
+#define ICONV_INCAST(p) (p)
+#endif
+#endif
+#ifndef ICONV_INCAST
+#define ICONV_INCAST(p) const_cast<char**>(p)
+#endif
+
 std::u16string toUtf16(std::string data) {
     static auto cached = iconv_cache("UTF-16LE", "UTF-8");
     static auto cachedBE = iconv_cache("UTF-16BE", "UTF-8");
@@ -73,7 +89,7 @@ std::u16string toUtf16(std::string data) {
     char *pout = reinterpret_cast<char*>(outbuffer.data());
     size_t inlen = data.size() + 1;
     size_t outlen = outbuffer.size() * sizeof (char16_t);
-    size_t retval = iconv(conv, const_cast<char**>(&pin), &inlen, &pout, &outlen);
+    size_t retval = iconv(conv, ICONV_INCAST(&pin), &inlen, &pout, &outlen);
     REQUIRE(retval != static_cast<size_t>(-1));
     iconv_close(conv);
     if (outlen > 0 && outbuffer[0] == 0xFEFF /* BOM */) {
@@ -100,7 +116,7 @@ std::u32string toUtf32(std::string data) {
     char *pout = reinterpret_cast<char*>(outbuffer.data());
     size_t inlen = data.size() + 1;
     size_t outlen = outbuffer.size() * sizeof (char32_t);
-    size_t retval = iconv(conv, const_cast<char**>(&pin), &inlen, &pout, &outlen);
+    size_t retval = iconv(conv, ICONV_INCAST(&pin), &inlen, &pout, &outlen);
     REQUIRE(retval != static_cast<size_t>(-1));
     iconv_close(conv);
     if (outlen > 0 && outbuffer[0] == 0xFEFF /* BOM */) {
@@ -127,7 +143,7 @@ std::u32string toUtf32(std::u16string data) {
     char *pout = reinterpret_cast<char*>(outbuffer.data());
     size_t inlen = (data.size() + 1) * 2;
     size_t outlen = outbuffer.size() * sizeof (char32_t);
-    size_t retval = iconv(conv, const_cast<char**>(&pin), &inlen, &pout, &outlen);
+    size_t retval = iconv(conv, ICONV_INCAST(&pin), &inlen, &pout, &outlen);
     REQUIRE(retval != static_cast<size_t>(-1));
     iconv_close(conv);
     if (outlen > 0 && outbuffer[0] == 0xFEFF /* BOM */) {
@@ -209,8 +225,12 @@ private:
 
 template <typename STR>
 Catch::Generators::GeneratorWrapper<std::vector<STR>> partitions(STR str) {
+#ifdef CATCH3
+    return Catch::Generators::GeneratorWrapper<std::vector<STR>>(new PartitionGenerator<STR>(str));
+#else
     return Catch::Generators::GeneratorWrapper<std::vector<STR>>(
         std::unique_ptr<Catch::Generators::IGenerator<std::vector<STR>>>(new PartitionGenerator<STR>(str)));
+#endif
 }
 
 TEST_CASE("test for PartitionGenerator", "[measurement]") {
@@ -385,7 +405,7 @@ TEST_CASE( "Measurements for single clusters", "[measurement]") {
         auto partition = GENERATE_COPY(partitions(testCase.str));
         INFO(testCase.desc);
         INFO("Partition: " << printPartition(partition));
-        INFO("Checking for string " << testCase.str)
+        INFO("Checking for string " << testCase.str);
         result = measureOneCluster(partition);
         CHECK(result.limitReached == true);
         CHECK(result.columns == testCase.columns);
@@ -394,7 +414,7 @@ TEST_CASE( "Measurements for single clusters", "[measurement]") {
     }
     SECTION("parse as utf16") {
         INFO(testCase.desc);
-        INFO("Checking for string " << testCase.str)
+        INFO("Checking for string " << testCase.str);
         std::u16string utf16 = toUtf16(testCase.str);
         auto partition = GENERATE_COPY(partitions(utf16));
         INFO("Partition: " << printPartition(partition));
@@ -406,7 +426,7 @@ TEST_CASE( "Measurements for single clusters", "[measurement]") {
     }
     SECTION("parse as utf32") {
         INFO(testCase.desc);
-        INFO("Checking for string " << testCase.str)
+        INFO("Checking for string " << testCase.str);
         auto partition = GENERATE_COPY(partitions(utf32));
         INFO("Partition: " << printPartition(partition));
         result = measureOneCluster(partition);
